%
% Este ficheiro contem um texto em Literate Haskell
%    Partes do ficheiro so texto em LaTeX e outras em Haskell
%    
%  Advanced Functional Programming
%  Universidade do Minho
%  2003/2004
%


\documentclass[12pt]{article}
\usepackage{moreverb}
\usepackage{a4wide}
\usepackage{graphics}
\usepackage{graphicx}
\usepackage{color}
\usepackage{alltt}
\usepackage[portuges]{babel}


\usepackage[latin1]{inputenc}
\usepackage{latexsym}


\usepackage[dvips]{epsfig}
\usepackage{epic}
\usepackage{eepic}


\def\Ipe#1{\def\IPEfile{#1}\input{#1}}

\newenvironment{code}
{\medskip\par\noindent\hrulefill 
\nopagebreak
\begin{center}
\begin{minipage}{\textwidth} 
\begin{alltt}\small}
{\end{alltt}
\end{minipage}
\end{center}
\hrule\medskip\noindent
}


\newtheorem{exercicio}{Exercise}[section]

\title{\sf\huge Advanced Functional Programming \\
\begin{tabular}{c}
{\small LMCC \& LESI}, {\small Universidade do Minho} \\
Strafunski: Exercises \\ 
\end{tabular}
}

\author{}
\date{}


%-------------------- Inicio do Documento -------------------------------

\begin{document}

\maketitle

\section{Preliminaries}

The subdirectory \texttt{StrategyLib} contains (version 4.0 of) Strafunski's strategy library. We assume you have either Hugs or GHC installed.

\section{Your first strategic program}

To get acquainted with the basics of strategic programming you will construct your first strategic program in a few simple steps.

\subsection{Load the library}

\begin{exercicio}
Create the file \texttt{MyFirstStrategicProgram.hs} with the following initial content:
\begin{center}\small
\begin{boxedverbatim}
module MyFirstStrategicProgram where
import StrategyLib
\end{boxedverbatim}
\end{center}
Load this program into Hugs or GHCi as follows:
\begin{center}\small
\begin{boxedverbatim}
ghci -fglasgow-exts -fallow-overlapping-instances \
     -fallow-undecidable-instances -package data \
	 -iStrategyLib-4.0/library:StrategyLib-4.0/models/drift-default: \
	 MyFirstStrategicProgram
\end{boxedverbatim}
\end{center}
\begin{center}\small
\begin{boxedverbatim}
hugs +o -98 +N  \
     -PStrategyLib-4.0/library:StrategyLib-4.0/models/drift-default: \
     MyFirstStrategicProgram
\end{boxedverbatim}
\end{center}
If all is well, the strategy library will be loaded into the interpreter, ready for use. With the \texttt{:t} command, you can ask the interpreter for the types of the library's combinators. Try this for \texttt{idTP}, \texttt{applyTP}, \texttt{adhocTP}, and \texttt{topdown}.
\begin{center}\small
\begin{boxedverbatim}
MyFirstStrategicProgram> :t idTP
idTP :: ...
MyFirstStrategicProgram> :t applyTP
applyTP :: ...
MyFirstStrategicProgram> :t adhocTP
adhocTP :: ...
MyFirstStrategicProgram> :t topdown
topdown :: ...
\end{boxedverbatim}
\end{center}
You will use these combinator in the next exercise.
\end{exercicio}
In these types, you see occurances of the type constructor \texttt{TP}, and the type class \texttt{Term}. The constraint \texttt{Term a} basically means: strategic functionality is activated for type \texttt{a}. If you are using Hugs, the command \texttt{:i Term} will reveal that this is the case for most of Haskell's predefined types. The type \texttt{TP m}, where \texttt{m} is a monad, is the type of (monadic) strategic functions that takes a term of any type \texttt{x} into a result of type \texttt{x}, i.e. it is a \emph{type-preserving generic function}.

\subsection{A type-preserving strategic function}

Next, you will construct a simple type-preserving strategic function using the combinators mentioned above.
\begin{exercicio}
Add the following import and function declaration to your first strategic program:
\begin{center}\small
\begin{boxedverbatim}
import MonadIdentity

increment :: Term a => a -> a
increment = runIdentity . applyTP strategy
  where 
    strategy = topdown (adhocTP idTP inc)
    inc x    = return (x + 1::Int)
\end{boxedverbatim}
\end{center}
As the type indicates, the function \texttt{increment} works on terms of any type, and maps them to a result of the same type. Apply it to terms of several types, and inspect the results:
\begin{center}\small
\begin{boxedverbatim}
MyFirstStrategicProgram> increment True
...
MyFirstStrategicProgram> increment (42::Int)
...
MyFirstStrategicProgram> increment [42::Int]
...
MyFirstStrategicProgram> increment ([42::Int],True,Just (7::Int))
...
\end{boxedverbatim}
\end{center}
As you can see, \texttt{increment} returns its argument unchanged, except that 1 has been added to any \texttt{Int} that occurs in it. The shape of the term, and the place where the \texttt{Int}s occur in it, are not important, and remain unchanged. 
\end{exercicio}
Note that, if \texttt{increment} had been an ordinary parametric polymorphic function of type \texttt{a -> a}, it could only have been the identity function. But the constraint \texttt{Term a} indicates that \texttt{increment} is not paramteric polymorphic, but that it is a \emph{strategic function}, which exhibits a mixture of generic and type-specifc behaviour. The generic behaviour is the identity, as expressed with the \texttt{idTP} combinator. The type specific behaviour is adding 1, as expressed by the local \texttt{inc} function. The \texttt{adhocTP} combinator combines these two into a single generic function with mixed behaviour. Finally, the \texttt{topdown} combinator applies this mixed behaviour to all subterms in a top-down fashion.

\subsection{Traversal variations}

The \texttt{topdown} combinator is a synonym for the \texttt{full\_tdTP} combinator. As the latter name indicates, it performs a \emph{full} traversal, i.e.\ it visits every subterm, in a top-down fashion.
\begin{exercicio}
Inspect the types of the following combinators:
\begin{center}\small
\begin{boxedverbatim}
MyFirstStrategicProgram> :t full_tdTP
full_tdTP :: ...
MyFirstStrategicProgram> :t full_buTP
full_buTP :: ...
MyFirstStrategicProgram> :t stop_tdTP
stop_tdTP :: (MonadPlus m) => ...
MyFirstStrategicProgram> :t once_tdTP
once_tdTP :: ...
MyFirstStrategicProgram> :t once_buTP
once_buTP :: ...
\end{boxedverbatim}
\end{center}
Note the occurrance of \texttt{MonadPlus} in the context of some of these combinators. An example of \texttt{MonadPlus} is the \texttt{Maybe} type constructor. It is used to indicate that a strategic function succeeds (\texttt{Just a}) or fails (\texttt{Nothing}).

To see these traversal variations in action, we will use a tree datatype \texttt{Tree} defined in the module \texttt{Tree} as:
\begin{center}\small
\begin{boxedverbatim}
data Tree = Tree Int [Tree]
\end{boxedverbatim}
\end{center}
Add an \texttt{import Tree} to your first strategic program, as well as the following function:
\begin{center}\small
\begin{boxedverbatim}
flip :: Term a => (TP Maybe -> TP Maybe) -> a -> a
flip traversal a = maybe a id (applyTP strategy a)
  where 
    strategy = traversal (adhocTP failTP flp)
    flp (Tree i []) = mzero
    flp (Tree i ts) = return (Tree i (reverse ts))
\end{boxedverbatim}
\end{center}
The \texttt{flip} function takes as first argument the \texttt{traversal} combinator to be applied. The type-specific behaviour defined by the local \texttt{flp} function is the reversal of the list of subtrees. The strategy combinator \texttt{failTP} is the dual of \texttt{idTP}: it fails to produce a result for any input term.

Now, apply flip to a tree, using various traversal combinators as arguments:
\begin{center}\small
\begin{boxedverbatim}
MyFirstStrategicProgram> flip once_tdTP myTree
...
MyFirstStrategicProgram> flip once_buTP myTree
...
MyFirstStrategicProgram> flip stop_tdTP myTree
...
MyFirstStrategicProgram> flip full_tdTP myTree
Tree 1 [Tree 2 [Tree 3 [],Tree 4 []],Tree 5 [Tree 6 [],Tree 7 []]]
MyFirstStrategicProgram> flip (full_tdTP.tryTP) myTree
...
MyFirstStrategicProgram> flip (full_buTP.tryTP) myTree
...
\end{boxedverbatim}
\end{center}
For example, define \texttt{myTree} as follows:
\begin{center}\small
\begin{boxedverbatim}
myTree = Tree 1 [Tree 2 [Tree 3 [],Tree 4 []],Tree 5 [Tree 6 [],Tree 7 []]]
\end{boxedverbatim}
\end{center}
Looking at the results, you should be able to understand the behaviour of each traversal combinator. Consult the online Haddock documentation of Strafunski (TraversalTheme) to verify your understanding.
\end{exercicio}
Note the use of \texttt{tryTP}. This combinator attempts to apply its argument strategy. If this fails, it does not propagate the failure, but returns the input term. We use \texttt{tryTP} in combination with the full traversals, since these need their argument strategies to succeed on every term to succeed themselves.

\subsection{A type-unifying strategic function}

So far, we have seen only one side of Strafunski: type-preserving strategies. To complement these, the library also offers type-unifying strategies.

\begin{exercicio}
Inspect the types of the following combinators:
\begin{center}\small
\begin{boxedverbatim}
MyFirstStrategicProgram> :t constTU
constTU :: ...
MyFirstStrategicProgram> :t applyTU
applyTU :: ...
MyFirstStrategicProgram> :t adhocTU
adhocTU :: ...
MyFirstStrategicProgram> :t collect
collect :: ...
MyFirstStrategicProgram> :t crush  
crush :: ...
MyFirstStrategicProgram> :t full_tdTU
full_tdTU :: ...
MyFirstStrategicProgram> :t once_tdTU
once_tdTU :: ...
MyFirstStrategicProgram> :t once_buTU
once_buTU :: ...
\end{boxedverbatim}
\end{center}
Consult the online Haddock documentation of Strafunski to find out what these combinators do.
\end{exercicio}
The type \texttt{TU a m}, where \texttt{m} is a monad, is the type of (monadic) strategic functions that takes a term of any type \texttt{x} into a result of the designated type \texttt{a}, i.e. it is a \emph{type-unifying generic function}. While type-preserving strategies perform \emph{transformations} of terms, type-unifying strategies are used to program \texttt{queries} of terms.

\begin{exercicio}
Add the following function definition to your first strategic program:
\begin{center}\small
\begin{boxedverbatim}
list :: Term a => a -> [Int]
list = runIdentity . applyTU strategy
  where 
    strategy = collect (adhocTU (constTU []) lst)
    lst x    = return [x]
\end{boxedverbatim}
\end{center}
As you can see, this function employs type-unifying strategy combinators. The type-specific behaviour defined by the local function \texttt{lst} is to put an integer into a singleton list. The generic behaviour of \texttt{constTU []} is to return an empty list. The collect strategy takes care of traversing the term in bottom-up fashion and appending all the singleton and empty lists into a final result.

Try the \texttt{list} function on various terms of various types. Create variations on \texttt{list} that employ different type-unifying traversal strategies, and test them on various terms.
\end{exercicio}

\medskip
\noindent
So far, we have applied our strategies on terms built from Haskell's predefined types and the simple user-defined \texttt{Tree} type. But the power of strategic programming is best experienced when processing terms build from large sets of mutually recursive datatypes. Examples of these are language syntaxes and file formats. The following sections demonstrate this on the abstract syntax of Haskell itself, and on XML document trees.

\section{Refactoring of Haskell Programs}

One of the examples included in the Strafunski distribution is a
program \verb@HsTransform@ that transforms haskell source code. It
performs two simple transformations: the elimination of the do
constructor and the introduction of a new type definition. You can run
the program (in hugs) as follows:

\begin{alltt} \footnotesize
runhugs  +o -98 +N
 -P<StrategyLib>/examples/haskell:<StrategyLib>/library:<StrategyLib>/models/drift-default: 
 HsTransform <inputfile> <outputfile>
\end{alltt}

\noindent
where \verb+StrategyLib+ is the location where you installed
Strafunski's StrategyLib (version 4.0-beta), and \verb+inputfile+ and
\verb+outputfile+ are the names of the Haskell program to be
transformed, and the file where you want the transformed program to be
written to.


\begin{exercicio} 

Write a little test program that makes use of various \verb+do+
constructions. Run the transformation program on your test program and
inspect the resulting program. Run your test program as well as the
transformed program to see if they behave the same.

Note: Be sure to test nested do-expression and monadic let-expressions
inside do's.

\end{exercicio}

\noindent
For instance, consider the following example Haskell program:

\begin{code}
Put your input test module here
\end{code}

Running the \verb+HsTransform+ program on it produces the following output:

\begin{code}
Put the transformed module here
\end{code}

\noindent
The elimination of \verb+do+ expressions is defined in the Haskell
Report, in section 3.14. This transformation is expressed in
Strafunski as follows:


\begin{code}
doElim 	   :: (Term a, MonadPlus m) => a -> m a
doElim h   =  applyTP (innermost (monoTP doElim1)) h

doElim1    :: MonadPlus m => HsExp -> m HsExp
doElim1 (HsDo [HsQualifier e]) 	
  = return e
doElim1 (HsDo (HsQualifier e:stmts))	
  = return (HsInfixApp e (HsVar (hsSymbol ">>")) (HsDo stmts))
doElim1 (HsDo (HsGenerator p e:stmts))
  = do ok <- new_name
       return (letPattern ok p e stmts)
doElim1 (HsDo (HsLetStmt decls:stmts))
  = return (HsLet decls (HsDo stmts))
doElim1 _                      
  = mzero
\end{code}


The function \verb+doElim1+ implements a single \verb+do+-elimination
rewrite step. It follows the definition in the Haskell Report quite
closely, except that instead of Haskell's concrete surface syntax,
it's abstract syntax is used.

The function \verb+doElim+ completes the rewrite step \verb+doElim1+
into a complete transformation. The \verb+monoTP+ combinator lifts the
rewrite step to a \emph{generic} rewrite step, and the
\verb+innermost+ combinator applies this generic rewrite step
repeatedly according to the innermost traversal strategy, until a
fixpoint is reached (no more redexes remain).




\begin{exercicio} 

In section 3.6 of the Haskell report a translation of the
\verb@if_then_else@ construct is presented. Implement this translation
in a strategic function named \verb@ifElim@. In order to incorporate
this transformation in the distributed package, just modify the Main
module to import module \verb+IfElim+, and modify one line in the main
function into:

\begin{verbatim}
      iowrap (mpipe [doElim,data2newtypeConv,ifElim])
\end{verbatim}

\end{exercicio}

\noindent

\noindent
For instance, consider the following example Haskell program:

\begin{code}
module TestIfElim where
f a = if (a > 1)
      then a
      else -a
\end{code}

Running the \verb+HsTransform+ program on it produces the following output:

\begin{code}
module TestIfElim where
f a
  = case (a > 1) of
	True -> a
	False -> - a
\end{code}


This transformation is expressed in Strafunski as follows:

\begin{code}
module IfElim where

import Datatypes
import DatatypesTermInstances
import StrategyLib
import Monad

ifElim 	   :: (Term a, MonadPlus m) => a -> m a
ifElim h   =  ...

ifElim1    :: MonadPlus m => HsExp -> m HsExp

\end{code}


\begin{exercicio} 

Implement the list comprehension elimination according to the rules
defined in section 3.11 of the haskell report. (if you solve this
exercise please email Joost the solution to be included in the next
Strafunski distribution).

\end{exercicio}


\section{Querying and transforming XML Documents}

Using the XML support from the HaXml project, you can easily import XML
documents into Haskell, and export them from Haskell. If you can supply a DTD
for your documents, you can additionally, inside Haskell, marshall between a
generic representation of XML documents (verified only) and a typed Haskell
representation of XML documents (validated by the Haskell type system).

As a running example, we will use documents that contain course descriptions.
An example document is given in \verb+teste.xml+, and its DTD is given in
\verb+Curso.dtd+.

To get things running you need to run \verb+DtdToHaskell+ on the given DTD to
obtain a file \verb+Curso.hs+ containing the Haskell datatypes that will
represent your document. Additionally, this file will contain instances of the
\verb+XmlContent+ class that provides the marshalling functionality.

\begin{exercicio} 
Run \verb+DtdToHaskell+ and inspect the content of the resulting file. Try to
understand the relationship between the DTD and the Haskell types, and between
XML documents and Haskell terms.
\end{exercicio}

\noindent
After generating the code, we can define some wrapper functions for performing
queries and transformations on given XML documents:

\begin{code}
module XMLExercise where

import Curso
import Text.XML.HaXml.Parse (xmlParse)
import Text.XML.HaXml.Types (Document(..),Content(..))
import Text.XML.HaXml.Xml2Haskell (showXml,fromElem,XmlContent(..))

processQ q xmlFile
  = do xmlInput <- readFile xmlFile
       let curso = fromString xmlInput :: Curso
       result <- q curso
       putStrLn (show result)

processT t xmlFile
  = do xmlInput <- readFile xmlFile
       let curso = fromString xmlInput :: Curso
       result <- t curso
       putStrLn (showXml result)

-- Helper for parsing

fromString :: XmlContent a => String -> a
fromString s 
  = forceResult (fst (fromElem [getElem (xmlParse "hoi" s)]))
    where
      getElem (Document _ _ e) = CElem e
      forceResult Nothing  = error "XML PARSE ERROR"
      forceResult (Just r) = r
\end{code}


\begin{exercicio}
Run a minimal query and a minimal transformation on the given example document
\verb+teste.xml+ to see if everything is working properly. Use \verb+ghci+ as
follows:
\begin{verbatim}
ghci -cpp -fglasgow-exts -fallow-overlapping-instances\
 -fallow-undecidable-instances -package data \
  -i<StrategyLib>/library:<StrategyLib>/models/drift-default:
  -i<HaXml>/src: XMLExercise
\end{verbatim}
where \verb+<HaXml>+ is the place where you installed HaXml. 
At the prompt, try:
\begin{verbatim}
*Main> processQ (const . putStrLn $ "Nada") "teste.xml"
*Main> processT return "teste.xml"
\end{verbatim}
Make sure you understand what is going on. For instance, do you understand why
the type annotation (\verb+:: Curso+) is used?
\end{exercicio}

If we make use of HaXml's marshalling functionality, writing queries and
transformations on XML documents can be done by plain functional programming on
the Haskell terms that represent these documents. But, since these documents
can contain large numbers of different element tags and attributes, plain
functional programming may quickly turn out to be clumsy.

\begin{exercicio}
Program a query that extracts all student names from a given XML document,
using standard functional programming techniques. Run the query with
\verb+processQ+.
\end{exercicio}

The following code shows how we can extract student names from a given XML
document using a set of recursive functions:

\begin{code}
studentNames
 = processQ (return . qCurso) "teste.xml"
   where
     qCurso (Curso _ _ cadeiras) 
       = ...
     qCadeira (Cadeira _ _ _ inscritos _) 
       = ...
     qInscritos (Inscritos _ alunos)
       = ...
     qAluno (Aluno _ nome)
       = nome
\end{code}

\noindent
Note that the recursive functions do pattern matching on all the parent types
of \verb+Aluno+. If the definition of \verb+Curso+, \verb+Cadeira+, or
\verb+Inscritos+ is changed (for instance by allowing further elements inside
them), the code above will need to be adapted as well.

Using the strategic function combinators of Strafunski, we can improve on this
situation. To use Strafunski, we need to generate instances of the \verb+Term+
class from the datatypes in \verb+Curso.hs+, using \verb+DrIFT+.

\begin{exercicio}
Generate the file \verb+CursoTermInstances.hs+ with an invocation of
\verb+DrIFT+, as follows:
\begin{verbatim}
> echo "module CursoTermInstances where" > CursoTermInstances.hs
> echo "import Curso" >> CursoTermInstances.hs
> echo "import TermRep" >> CursoTermInstances.hs
> DrIFT -g Term -g Typeable -r Curso.hs >> CursoTermInstances.hs
\end{verbatim}
The instances of class \verb+Term+ and \verb+Typeable+ are used only internally
by the strategy combinators of Strafunski. As a user of these combinators, you
do not need to understand them.
\end{exercicio}

Using Strafunski, the same query can be implemented more concisely as follows:

\begin{code}
...
import StrategyLib
import CursoTermInstances
...

studentNames'
  = processQ (applyTU q) "teste.xml"
    where
      q = stop_tdTU (monoTU qAluno) 
      qAluno (Aluno _ nome) = return [nome]
\end{code}

\noindent The traversal strategy \verb+stop_tdTU+ is used to search for
students in a top down fashion, without continuing the search on the
subelements of a student. The \verb+monoTU+ combinator turns the monomorphic
function \verb+qAluno+ into a generic function that succeeds on students, but
fails on other elements.

\begin{exercicio}
Verify that the reformulated query produces the same results as the original
one. Make sure you understand the reformulated query.
\end{exercicio}

Apart from querying the document, we can transform it. For instance, we can
update the attribute \verb+n+ of the element \verb+Inscritos+ such that we are
sure that it matches the actual number of students registred for the course.

\begin{exercicio}
Complete the following code. And run it on the document \verb+teste.xml+.
\end{exercicio}

The following code traverses the document to find elements of type
\verb+Inscritos+. These elements are subsequently queried to find the number of
students that it holds. Finally, the attribute of the element \verb+Inscritos+
is updated to the computed number of students.

\begin{code}
countAlunos 
 = applyTU ( ... )
   where
     countAluno (a::Aluno) = return (1::Int)

updateInscritosN
  = applyTP( ... )
    where
      update (Inscritos attrs alunos)
        = do n <- countAlunos alunos
             let attrs' = attrs{inscritosN=Just (show n)}
             return (Inscritos attrs' alunos)
\end{code}

\noindent
Note the use of record update to perform attribute updating.

\end{document}
